import { HoliEvent } from './fetch_events.ts'
import { logger } from './logging.ts'
import { Activity } from './providers/jasd/appapi.dto.types.ts'
import NotFound = Deno.errors.NotFound

const APP_API_BASE_URL = 'https://gemeinschaftswerk-nachhaltigkeit.de/app/api/v1'
const APP_FILES_BASE_URL = 'https://gemeinschaftswerk-nachhaltigkeit.de/app/api/v1/files'

export type HoliEventResponse = {
  data: HoliEvent
}

export type FetchEventInput = {
  id: string
}

export const toHoliEvent = (activity: Activity): HoliEvent | null => {
  if (activity.period.permanent) return null

  return {
    id: activity.id.toString(),
    title: activity.name,
    organisationName: activity.organisation.name,
    startDate: activity.period.start,
    endDate: activity.period.end,
    location: {
      city: activity.location.address.city,
      street: activity.location.address.street,
      streetNo: activity.location.address.streetNo,
      zipCode: activity.location.address.zipCode,
    },
    isRemote: activity.location.online,
    // todo gregor: imageproxy-fi the url
    imageUrl: APP_FILES_BASE_URL + '/' + activity.image,
    eventDetails: {
      description: activity.description,
      externalLink: activity.location.url,
      meetingLink: activity.registerUrl,
    },
  }
}

export const fetchEvent = async (input: FetchEventInput) => {
  const url = new URL(`${APP_API_BASE_URL}/activities/${input.id}`)
  const startDate = Date.now()

  logger.info(`fetching event from ${url}`)

  const response = await fetch(url)

  if (response.status === 404) {
    throw new NotFound('Not found')
  }

  try {
    const json = (await response.json()).activity as Activity

    const holiEvent = toHoliEvent(json)

    return {
      data: holiEvent,
    }
    // deno-lint-ignore no-explicit-any
  } catch (e: any) {
    logger.error(
      `Error performing request to ${url}: ${e.message}`,
    )
    throw e
  } finally {
    const duration = Date.now() - startDate
    logger.debug(`fetching projects took ${duration} ms`)
  }
}
