import { logger } from './logging.ts'
import { AppApiEvent, AppApiResponse } from './providers/jasd/appapi.dto.types.ts'

//const OPEN_API_BASE_URL = 'https://gemeinschaftswerk-nachhaltigkeit.de/app/openapi/v1'
const APP_API_BASE_URL = 'https://gemeinschaftswerk-nachhaltigkeit.de/app/api/v2'
const APP_FILES_BASE_URL = 'https://gemeinschaftswerk-nachhaltigkeit.de/app/api/v1/files'

export type HoliEventLocation = {
  street?: string
  streetNo?: string
  zipCode?: string
  city?: string
}

export type HoliEventDetails = {
  description: string
  externalLink: string
  meetingLink?: string
}

export type HoliEvent = {
  id: string
  title: string
  organisationName: string
  startDate: string | null
  endDate: string | null
  location: HoliEventLocation
  isRemote: boolean
  imageUrl: string
  eventDetails: HoliEventDetails
}

export type HoliEventsResponse = {
  totalResults: number
  data: HoliEvent[]
}

const toHoliEvent = (event: AppApiEvent): HoliEvent | null => {
  if (event.activity.period.permanent) return null

  return {
    id: event.activity.id.toString(),
    title: event.name,
    organisationName: event.activity.organisation.name,
    startDate: event.activity.period.start,
    endDate: event.activity.period.end,
    location: {
      city: event.location.address.city,
      street: event.location.address.street,
      streetNo: event.location.address.streetNo,
      zipCode: event.location.address.zipCode,
    },
    isRemote: event.activity.location.online,
    // todo gregor: imageproxy-fi the url
    imageUrl: APP_FILES_BASE_URL + '/' + event.activity.image,
    eventDetails: {
      description: event.description,
      externalLink: event.activity.location.url,
      meetingLink: event.activity.registerUrl,
    },
  }
}
/*
const buildJasdOpenApiUrl = (
  { limit, offset, location }: FetchEventsInput,
): URL => {
  const page = Math.floor(offset / limit) + 1 // page is NOT zero-based (!), i.e. first page = 1
  const url = new URL(`${OPEN_API_BASE_URL}/activities/events`)
  url.searchParams.append('page', page.toString())
  url.searchParams.append('size', limit.toString())

  //todo: gregor doesn't work in the open api
  if (location) {
    url.searchParams.append('location', location)
  }
  return url
}
*/

const buildJasdAppApiUrl = (
  { limit, offset, location }: FetchEventsInput,
): URL => {
  const url = new URL(`${APP_API_BASE_URL}/search`)
  url.searchParams.append('page', offset.toString())
  url.searchParams.append('size', limit.toString())
  url.searchParams.append('sort', 'period.start,asc,ignorecase')
  // filtering outdated and permanent events here helps us a lot
  // and provides only OnlineTimedEvent and RealTimedEvent
  url.searchParams.append('includeExpiredActivities', 'false')
  url.searchParams.append('permanent', 'false')

  if (location) {
    url.searchParams.append('location', location)
  }
  return url
}

type EventsPage = {
  events: HoliEvent[]
  onLastPage: boolean
}

const fetchEventsPage = async (
  params: FetchEventsInput,
): Promise<EventsPage> => {
  const url = buildJasdAppApiUrl(params)

  const startDate = Date.now()

  logger.info(`fetching projects from ${url}`)
  try {
    const response = await fetch(url, {
      // gregor: we don't need to send the api key to the App api
      // only the OpenApi endpoints require auth
      // headers: { 'x-api-key': Deno.env.get('JASD_DEVELOPER_API_KEY')! },
    })
    const json = await response.json() as AppApiResponse

    const holiEvents = json.content
      .flat()
      .map(toHoliEvent)
      .filter((holiEvent) => !!holiEvent)

    return {
      events: holiEvents,
      onLastPage: json.last,
    }
    // deno-lint-ignore no-explicit-any
  } catch (e: any) {
    logger.error(
      `Error performing request to ${url}: ${e.message}`,
    )
    throw e
  } finally {
    const duration = Date.now() - startDate
    logger.debug(`fetching projects took ${duration} ms`)
  }
}

export type FetchEventsInput = {
  limit: number
  offset: number
  location?: string
}

export const fetchEvents = async (
  { offset, limit, location }: FetchEventsInput,
): Promise<HoliEventsResponse> => {
  const holiEvents = []
  let hasMoreEvents = true
  let attempt = 0

  while (holiEvents.length < limit && hasMoreEvents) {
    const fetchResult = await fetchEventsPage({
      offset: offset + attempt,
      limit,
      location,
    })

    holiEvents.push(...fetchResult.events)
    hasMoreEvents = !fetchResult.onLastPage
    attempt++
  }

  return {
    totalResults: holiEvents.length,
    data: holiEvents,
  }
}
